/*

------------
Introduction
------------
This script can be used to create a bootable CD with enough facilities
to fully manage and recover your system.

It contains support for LVM, HPFS, JFS, FAT32 and CD.  The total space
occupied is less than five MB!

Personally I use it on the CD-RWs I burn my system backups to.
Recovering/reinstalling the system just takes booting-off this CD,
optionally managing (lvm) and formatting volumes, and unzipping the
backup-zip files that are also on the CD!

Present settings assume an IDE only setup, instuctions for adding SCSI
support are included in the configuration file.  If you need extra
space on the boot image (for instance if you added SCSI support),
instructions for moving some files to the CD are included as well.
Full USB support is also enabled by and defaults to one UHCI adapter.
Of course this can be fully configured as well.

The script assumes you use RSJ for burning CDs and that you have
already attached the CD-R(W).  A virtual 2.88MB floppy should also be
available.  Instructions for using this script with cdrecord and
mkisosoft are included as well.

The script furthermore assumes you use the DANI drivers for accessing
IDE and ATAPI devices as these are the most stable. This can be
configured, however.

------------
Installation 
------------
Installation is simple, just copy the files to the correct locations:
    bootCD.cmd   should go to a directory on the PATH
    bootCD.cfg   should go to a directory on the DPATH
    daniboot.flt should go to a directory on the DPATH

Instead of copying the files to locations on the PATH/DPATH, a
specific directory where all files reside will of course also work
(just make sure the command is invoked from that directory...)

See below for instructions on where to get the other required files.

------
Syntax
------
bootcd [<configuration file>]

By default the configuration file used is bootCD.cfg located on the
DPATH.

------------------------
Creating a bootable disk
------------------------
Here's a few easy steps for creating your first bootable disk.

1) Download and install all required files.

2) Edit the configuration file bootCD.cfg:
   Configuration of the script is via the bootCD.cfg file.  The
   options are all commented.  Work through them top to bottom, most
   likely you will only need/want to change a few of the 'basic'
   ones.  You'll probably not going to neet to bother with the
   'advanced' ones (unless you don't use the DANI drivers AND you
   know what you are doing...
   Here are some hints for the first try:
   a) You probably need to change the directories/drives to match
      your setup.
   b) You probably want to specify BOOTMESSAGES="YES", VERBOSE=2,
        PAUSEBEFOREBOOTIMAGE="YES".
   c) Check the USB adapter type.
   d) If you have SCSI, set the SCSI option and specify the device
      driver for your card.  (You may need to set the TWODISKBOOT
      option or turn some other options off is this is a realy big
      file >134KB)
   e) Add a REM statement in front of the CALL statement of the
      ADDITIONALCFG option.
   This should cover most cases, time to create the first boot image!

3) Insert an empty CD-R(W) in your burner.  I would realy suggest to
   use CD-RW media; you are probably going to run this script a lot of
   times to get things the way you want...  I know CD-Rs are dirt
   cheap, but please give the environment a break and what would you
   do with tons of coasters?  (Yeah, you can microwave them for great
   effects but that becomes boring after a while as well :-)

4) Attach the CD-R(W) (cdattach w:).  If it's a CD-RW you can erase it
   as well (cdwfmt w: /erase).

5) Run the script (bootcd)

6) Put additional files on the CD if you like.

7) Burn the CD (cdattach w: -s).

8) Test it.

That's all!  Now you probably want to play a bit with the numerous
options :-)

-----------------------------
Using cdrecord instead of RSJ
-----------------------------
When using cdrecord and mkisofs, change the configuration variable
CDWRITER to the path you are going to build the CD in and set the
option CDRECORD to "YES".

The script is now ready to be used for cdrecord/mkisofs.  Just one
more thing needs to be done after running the script; instead of
creating a normal ISO image for burning by cdrecrord, you need to
specify CDBOOT_1.IMG as the boot image on the mkisofs command-line (-b
option to mkisofs).

------------------------
Obtaining required files
------------------------
All required files can be obtained from Hobbes:
    savedskf.exe: search for loaddf.zip
    daniboot.flt: search for daniboot
    danis506.add: search for danis506
    daniatap.flt: search for daniatapi
    fat32:        search for os2fat32
    virtual disk: search for vfdisk

Another option for a virtual disk is Super Virtual Disk.
This is shareware and can be obtained from bmtmicro

For info about RSJ, look at their website

------
Thanks
------
Thanks go to Allan Dermody for providing CD_BOOT.FLT and the
first insight into creating bootable CD's

Thanks also go to Alfrede Fernandez Diaz for writing an excelent
article on how to create a bootable CD in OS2Voice, July 2000
http://www.os2voice.org/VNL/past_issues/VNL0700H/vnewsf2.htm

More thanks go to Daniela Engert for writing the DANI suite of
device drivers and filters.

Enjoy!

-------------------
Contact information
-------------------
Author:
    Name:    Hayo Baan
    e-mail:  hrbaan@xs4all.nl
    updates: http://www.xs4all.nl/~hrbaan

RSJ:
    Website: http://www.rsj.de
    
BMT-Micro:
    Website: http://www.bmtmicro.com

Hobbes:
    Website: http://hobbes.nmsu.edu

---------------
Version History
--------------- 
v1.4 xx/08/2002: Files are copied without extended attributes
                 Added bootcd comman-line option for the specification
                   of the configuration file
                 Corrected error in option name in config.sys for fallback
                   driver (wasn't renamed)
                 USB printer device now defaults to LPTn instead of $USBPRT
                 Added parallel port (printer) support
                 Added serial port support
                 Added mouse support
                 Improved the way of to specificy which shell to use, now
                   it is possible to specify PROTSHELL and OS2_SHELL
                   independantly (advanced options)
                 Improved VERBOSE, you can now specify the message level
                 Added genral instructions for using this script
                 Added instructions for using this script with
                   cdrecord/mkisofs
                 Added ability to split boot image over two disks
                   providing even more room for device drivers and
                   other things
                 Other general improvements

v1.3 22/08/2002: Added USB support
                 Fixed bug with SHELLFILE variable not respected
                 Renamed option IDECD to ATAPI (more correct)
                 Renamed option HDDFALLBACK to DASDFALLBACK (more
                   correct)
                 Now possible to specify options for IFS and BASEDEV
                   lines
                 Added comment why there is no autocheck for IFSs
                 Better option handling, not case sensitive anymore,
                   specifying 'Y' is enough
                 Cache sizes for HPFS and JFS now 1024KB by default
                 Added VERBOSE option with this to "YES" all command output
                   is shown.
                 Added option to pause the script right before writing the
                   image to CD.  You can use this to check and or correct
                   the content of the boot image.
                 
v1.2 20/08/2002: Now a REXX script
                 Added configuration options
                 CONFIG.SYS is now created automatically by the script
                   and respects set options.
                 Added notes concerning virtual disk software
                 Added contact information

v1.1 21/02/2002: Now using daniboot.flt instead of cdboot.flt

v1.0 16/02/2002: Original version

*/

"@echo off"

/* Load the rexUtil functions                                              */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

/* String variable to insert a newline in a character string               */
CRLF=d2c(13)||d2c(10)

/* Determine temp directory                                                */
TEMPDIR=value("TEMP",,"OS2ENVIRONMENT")
if TEMPDIR="" then TEMPDIR=value("TMP",,"OS2ENVIRONMENT")
if right(TEMPDIR,1)<>"\" then TEMPDIR=TEMPDIR"\"

/* Read the configuration parameters                                       */
rc=readConfig(ARG(1))

/* Determine if command output should be shown                             */
if VERBOSE=9 then
    IGNOREOUTPUT=""
else
    IGNOREOUTPUT="> /dev/nul 2>&1"

/***************************************************************************/
/**** Boot Image ***********************************************************/
/***************************************************************************/

rc=prepareImage(1)
rc=copyImageFiles(1)
rc=createConfigSys()

if isSet(BOOTMESSAGES) then do
    rc=charout(VIRTUALDRIVE"\ALTF2ON.$$$","")
    rc=charout(VIRTUALDRIVE"\ALTF2ON.$$$")
end

if isSet(TWODISKBOOT) then do
    rc=writeImage(1)
    rc=prepareImage(2)
end

rc=copyImageFiles(2)

if isSet(TWODISKBOOT) then
    rc=writeImage(2)
else
    rc=writeImage(1)


/***************************************************************************/
/**** Support Files on CD **************************************************/
/***************************************************************************/

call progress 1, "Writing files to CD"

/* Files to copy to os2 directory of CD */
rc=createDir(WRITERDRIVE"\OS2")
FILES=EXTRAFILES
if isSet(MAKEROOM) then do
    FILES=FILES VITALFILES_ SHELLFILE ADDITIONALSHELLFILES
    if fileName(SHELLFILE)<>fileName(PROTSHELLFILE) then FILES=FILES PROTSHELLFILE
    if isSet(CDFS)         then FILES=FILES CDFSFILES
    if isSet(FAT)          then FILES=FILES FATFILES
    if isSet(HPFS)         then FILES=FILES HPFSFILES
    if isSet(JFS)          then FILES=FILES JFSFILES
    if isSet(LVM)          then FILES=FILES LVMFILES
    if isSet(PARALLELPORT) then FILES=FILES PARALLELPORTFILE
    if isSet(SERIALPORT)   then FILES=FILES SERIALPORTFILE
    if isSet(MOUSE)        then FILES=FILES MOUSEPOINTDDFILE MOUSEFILE MOUSEDEVFILE
    if isSet(USB) then do
        if isSet(USBKEYB)    then FILES=FILES USBKEYBFILE
        if isSet(USBCOM)     then FILES=FILES USBCOMFILE
        if isSet(USBMOUSE)   then FILES=FILES USBMOUSEFILE
        if isSet(USBPRINTER) then FILES=FILES USBPRINTERFILE
    end
end
rc=copyFiles(FILES, WRITERDRIVE"\os2")

/* Files to copy to os2\dll directory of CD */
rc=createDir(WRITERDRIVE"\OS2\DLL")
DLLS=EXTRADLLS
if isSet(MAKEROOM) then do
    if isSet(CDFS)    then DLLS=DLLS CDFSDLLS
    if isSet(FAT)     then DLLS=DLLS FATDLLS
    if isSet(HPFS)    then DLLS=DLLS HPFSDLLS
    if isSet(JFS)     then DLLS=DLLS JFSDLLS
    if isSet(LVM)     then DLLS=DLLS LVMDLLS
    if isSet(UNICODE) then DLLS=DLLS OSBOOTDRIVE"\os2\dll\uconv.dll"
end
rc=copyFiles(DLLS,  WRITERDRIVE"\os2\dll")

call progress 1, "Done creating bootable CD"

exit


/***************************************************************************/
/**** Functions ************************************************************/
/***************************************************************************/

/* Write an error message to stderr -------------------------------------- */
errorMessage:
    call lineout "STDERR", "ERROR:" ARG(1)
    return 0

/* Copy a list of files -------------------------------------------------- */
copyFiles:
    SOURCEFILES=ARG(1)
    DESTINATION=ARG(2)
    if right(DESTINATION,1)<>"\" then DESTINATION=DESTINATION"\"
    do while words(SOURCEFILES)>0
        rc=sysFileTree(word(SOURCEFILES,1), FILETREE, "F")
        if rc=0 then do
            do I=1 to FILETREE.0
                rc=copyFile(FILETREE.I, DESTINATION)
            end I
            if FILETREE.0=0 then
                call errorMessage "File not found" word(SOURCEFILES,1)
        end
        else
            call errorMessage "Not enough memory copying" word(SOURCEFILES,1)
        SOURCEFILES=delword(SOURCEFILES,1,1)
    end
    return rc


/* Copy a file without extended attributes ------------------------------- */
copyFile:
    parse value ARG(1) with dummy dummy dummy ATTRIBUTES SOURCEFILE
    ATTRIBUTES=space(translate(ATTRIBUTES, " ", "-"),0)
    SOURCEFILE=strip(SOURCEFILE)
    call progress 2, "Copying" left(ATTRIBUTES,4) SOURCEFILE "=>" ARG(2)
    "xcopy /h/t/r" SOURCEFILE TEMPDIR IGNOREOUTPUT
    select
        when rc=1 then ERR="File not found"
        when rc=2 then ERR="File errors"
        when rc=3 then ERR="Interrupted"
        when rc=4 then ERR="Not enough room for file"
        otherwise ERR=""
    end
    if rc=0 then do
        "attrib -ahrs" TEMPDIR||fileName(SOURCEFILE) IGNOREOUTPUT
        if rc<>0 then ERR="Attrib setting -ahrs failed ("rc")"
    end
    if rc=0 then do
        "eautil /s /r" TEMPDIR||fileName(SOURCEFILE) TEMPDIR||"eautil.$$$" IGNOREOUTPUT
        if rc<>0 then ERR="Eautil failed"
    end
    if rc=0 then do
        "move" TEMPDIR||fileName(SOURCEFILE) ARG(2) IGNOREOUTPUT
        if rc<>0 then ERR="Move to" ARG(2) "failed ("rc")"
    end
    if rc=0 & ATTRIBUTES<>"" then do
        "attrib +"ATTRIBUTES ARG(2)||fileName(SOURCEFILE) IGNOREOUTPUT
        if rc<>0 then ERR="Attrib setting +"ATTRIBUTES "failed ("rc")"
    end
    if rc=0 then do
        rc=sysFileDelete(TEMPDIR||"eautil.$$$")
        if rc=2 then rc=0
        if rc<>0 then ERR="Removing" TEMPDIR||"eautil.$$$ failed ("rc")"
    end
    if rc<>0 & translate(fileName(SOURCEFILE))<>"USER.LOC" then
        call errorMessage "Error copying" SOURCEFILE||":" ERR
    return rc


/* Create a directory ---------------------------------------------------- */
createDir:
    DIRNAME=ARG(1)
    rc=sysMkDir(DIRNAME)
    select
        when rc=2   then ERR="File not found"
        when rc=3   then ERR="Path not found"
        when rc=5   then ERR="Access denied"
        when rc=26  then ERR="Not a DOS disk"
        when rc=87  then ERR="Invalid parameter"
        when rc=108 then ERR="Drive locked"
        when rc=206 then ERR="Filename exceeds range"
        otherwise ERR=""
    end
    if rc<>0 then call errorMessage "Error creating directory" DIRNAME||":" ERR
    return rc


/* Write a line to the config.sys file ----------------------------------- */
configLine:
    rc=lineout(VIRTUALDRIVE"\config.sys",ARG(1))
    return rc


/* Return file name portion of a filespec -------------------------------- */
fileName:
    return filespec("NAME", ARG(1))


/* Read and parse the configuration file --------------------------------- */
readConfig:
    CONFIGFILE=ARG(1)
    if CONFIGFILE="" then CONFIGFILE=SysSearchPath("DPATH","bootcd.cfg")
    if STREAM(CONFIGFILE,"C","QUERY EXISTS")="" then do
        call errorMessage "Configuration file not found"
        exit 1
    end
    STATEMENT=""
    LINENR=0
    do while lines(CONFIGFILE)>0
        LINE=strip(linein(CONFIGFILE))
        LINENR=LINENR+1
        if length(LINE)>0 & left(LINE,2)<>"/*" then do
            CONT=0
            if right(LINE,1)="," then do
                CONT=1
                LINE=left(LINE,length(LINE)-1)
            end
            STATEMENT=strip(STATEMENT LINE)
            if CONT=0 then do
                interpret STATEMENT
                STATEMENT=""
            end
        end
    end
    return 0


/* Determine if an option is set ----------------------------------------- */
isSet:
    return (left(translate(ARG(1)),1)="Y")


/* Display progress when in given verbose mode --------------------------- */
progress:
    if VERBOSE>=ARG(1) then say ARG(2)
    return rc


/* Prepare a boot image -------------------------------------------------- */
prepareImage:
    IMAGENR=ARG(1)
    /* Format the virtual disk */
    call progress 1, "Formatting boot image" IMAGENR
    "format" VIRTUALDRIVE "/fs:fat /once /v:CDBOOT_"IMAGENR IGNOREOUTPUT
    if rc<>0 then call errorMessage "Error formatting boot image" IMAGENR
    if rc=0 then do
        /* Install system files */
        call progress 1, "Transfering system files to boot image" IMAGENR
        SYSINSTX VIRTUALDRIVE IGNOREOUTPUT
        if rc<>0 then call errorMessage "Error transfering system files to boot image" IMAGENR
    end
    return rc


/* Write a boot image to CD ---------------------------------------------- */
writeImage:
    IMAGENR=ARG(1)
    if isSet(PAUSEBEFOREBOOTIMAGE) then do
        say "Pausing before writing boot image" IMAGENR
        "pause"
    end

    call progress 1, "Writing boot image" IMAGENR
    "savedskf /a/d" VIRTUALDRIVE WRITERDRIVE"\CDBOOT_"IMAGENR".IMG" IGNOREOUTPUT
    if rc<>0 then call errorMessage "Error writing boot image" IMAGENR

    /* The first image will be marked bootable */
    if IMAGENR=1 & \ isSet(CDRECORD) then do
        "cdboot" WRITERDRIVE"\CDBOOT_"IMAGENR".img" IGNOREOUTPUT
        if rc<>0 then call errorMessage "Error marking boot image"
    end
    return rc


/* Copy files to boot image ---------------------------------------------- */
copyImageFiles:
    IMAGENR=ARG(1)

    call progress 1, "Copying files to boot image" IMAGENR

    /* Files to copy to root directory of boot image */
    if IMAGENR=1 | isSet(TWODISKBOOT) then
        rc=copyFiles(VITALROOTFILES.IMAGENR, VIRTUALDRIVE"\")

    /* Files to copy to os2 directory of boot image */
    if IMAGENR=1 | isSet(TWODISKBOOT) then
        rc=createDir(VIRTUALDRIVE"\OS2")
    if IMAGENR=2 then do
        FILES=VITALFILES
        if isSet(CDROM)   then FILES=FILES CDROMDMDFILE
        if isSet(CDFS)    then FILES=FILES CDFSIFSFILE
        if isSet(FAT)     then FILES=FILES FATIFSFILE
        if isSet(HPFS)    then FILES=FILES HPFSIFSFILE
        if isSet(JFS)     then FILES=FILES JFSIFSFILE
        if isSet(UNICODE) then FILES=FILES OSBOOTDRIVE"\os2\boot\unicode.sys"
        if \ isSet(MAKEROOM) then do
            FILES=FILES VITALFILES_ SHELLFILE ADDITIONALSHELLFILES
            if fileName(SHELLFILE)<>fileName(PROTSHELLFILE) then FILES=FILES PROTSHELLFILE
            if isSet(CDFS)         then FILES=FILES CDFSFILES
            if isSet(FAT)          then FILES=FILES FATFILES
            if isSet(HPFS)         then FILES=FILES HPFSFILES
            if isSet(JFS)          then FILES=FILES JFSFILES
            if isSet(LVM)          then FILES=FILES LVMFILES
            if isSet(PARALLELPORT) then FILES=FILES PARALLELPORTFILE
            if isSet(SERIALPORT)   then FILES=FILES SERIALPORTFILE
            if isSet(MOUSE)        then FILES=FILES MOUSEPOINTDDFILE MOUSEFILE MOUSEDEVFILE
            if isSet(USB) then do
                if isSet(USBKEYB)    then FILES=FILES USBKEYBFILE
                if isSet(USBCOM)     then FILES=FILES USBCOMFILE
                if isSet(USBMOUSE)   then FILES=FILES USBMOUSEFILE
                if isSet(USBPRINTER) then FILES=FILES USBPRINTERFILE
            end
        end
        rc=copyFiles(FILES, VIRTUALDRIVE"\os2")
    end
    
    /* Files to copy to os2\boot directory of boot image */
    if IMAGENR=1 then do
        rc=createDir(VIRTUALDRIVE"\OS2\BOOT")
        BOOTFILES=VITALBOOTFILES BOOTFLTFILE
        if isSet(FLOPPY)       then BOOTFILES=BOOTFILES FLOPPYADDFILE
        if isSet(DASDFALLBACK) then BOOTFILES=BOOTFILES DASDFALLBACKFILE
        if isSet(IDE)          then BOOTFILES=BOOTFILES IDEADDFILE
        if isSet(SCSI)         then BOOTFILES=BOOTFILES SCSIADDFILE
        if isSet(ATAPI)        then BOOTFILES=BOOTFILES ATAPIFLTFILE
        if isSet(ASPI)         then BOOTFILES=BOOTFILES ASPIDMDFILE
        if isSet(LVM)          then BOOTFILES=BOOTFILES LVMDMDFILE
        if isSet(USB) then do
            BOOTFILES=BOOTFILES USBDFILE USBHIDFILE
            if USBOHCI>0 then BOOTFILES=BOOTFILES USBOHCIFILE
            if USBUHCI>0 then BOOTFILES=BOOTFILES USBUHCIFILE
            if USBEHCI>0 then BOOTFILES=BOOTFILES USBEHCIFILE
            if isSet(USBCDROM)   then BOOTFILES=BOOTFILES USBCDROMFILE
            if isSet(USBSTORAGE) then BOOTFILES=BOOTFILES USBSTORAGEFILE
        end
        rc=copyFiles(BOOTFILES, VIRTUALDRIVE"\os2\boot")
    end
        
    /* Files to copy to os2\dll directory of boot image */
    if IMAGENR=2 then do
        rc=createDir(VIRTUALDRIVE"\OS2\DLL")
        DLLS=VITALDLLFILES
        if \ isSet(MAKEROOM) then do
            if isSet(CDFS)    then DLLS=DLLS CDFSDLLS
            if isSet(FAT)     then DLLS=DLLS FATDLLS
            if isSet(HPFS)    then DLLS=DLLS HPFSDLLS
            if isSet(JFS)     then DLLS=DLLS JFSDLLS
            if isSet(LVM)     then DLLS=DLLS LVMDLLS
            if isSet(UNICODE) then DLLS=DLLS OSBOOTDRIVE"\os2\dll\uconv.dll"
        end
        rc=copyFiles(DLLS, VIRTUALDRIVE"\os2\dll")
    end
    
    /* Unicode support files */
    if IMAGENR=2 & isSet(UNICODE) then do
        /* Language directory tree */
        rc=createDir(VIRTUALDRIVE"\LANGUAGE")
        rc=createDir(VIRTUALDRIVE"\LANGUAGE\CODEPAGE")
        rc=createDir(VIRTUALDRIVE"\LANGUAGE\"UNILANGDIR)
        rc=createDir(VIRTUALDRIVE"\LANGUAGE\FONT")
        rc=createDir(VIRTUALDRIVE"\LANGUAGE\KEYBOARD")
        rc=createDir(VIRTUALDRIVE"\LANGUAGE\LOCALE")
        rc=copyFiles(OSBOOTDRIVE"\language\codepage\"UNICP,      VIRTUALDRIVE"\language\codepage")
        rc=copyFiles(OSBOOTDRIVE"\language\"UNILANGDIR"\ehx*.*", VIRTUALDRIVE"\language\"UNILANGDIR)
        rc=copyFiles(OSBOOTDRIVE"\language\keyboard\"UNIKEYB,    VIRTUALDRIVE"\language\keyboard")
        rc=copyFiles(OSBOOTDRIVE"\language\locale\user.loc",     VIRTUALDRIVE"\language\locale")
    end
    
    return rc

    
/* Create config.sys file ------------------------------------------------ */

createConfigSys:
    call progress 1, "Creating config.sys file on boot image"
    
    /* Drive letter for files that can reside on boot image or on CD */
    DRIVELETTER="a:"
    if isSet(MAKEROOM) then DRIVELETTER="s:"
    
    /* CD-ROM = S: */
    rc=configLine("RESERVEDRIVELETTER=R")
    
    /* File systems */
    if isSet(CDFS) then rc=configLine("IFS=\os2\"fileName(CDFSIFSFILE) CDFSOPTIONS)
    if isSet(FAT)  then rc=configLine("IFS=\os2\"fileName(FATIFSFILE)  FATOPTIONS)
    if isSet(HPFS) then rc=configLine("IFS=\os2\"fileName(HPFSIFSFILE) HPFSOPTIONS)
    if isSet(JFS)  then rc=configLine("IFS=\os2\"fileName(JFSIFSFILE)  JFSOPTIONS)

    /* Base device drivers */
    rc=configLine("BASEDEV=ibmkbd.sys")
    rc=configLine("BASEDEV=os2dasd.dmd")
    
    if isSet(FLOPPY)       then rc=configLine("BASEDEV="fileName(FLOPPYADDFILE)    FLOPPYOPTIONS)
    if isSet(IDE)          then rc=configLine("BASEDEV="fileName(IDEADDFILE)       IDEOPTIONS)
    if isSet(SCSI)         then rc=configLine("BASEDEV="fileName(SCSIADDFILE)      SCSIOPTIONS)
    if isSet(ATAPI)        then rc=configLine("BASEDEV="fileName(ATAPIFLTFILE)     ATAPIOPTIONS)
    if isSet(DASDFALLBACK) then rc=configLine("BASEDEV="fileName(DASDFALLBACKFILE) DASDFALLBACKOPTIONS)
    if isSet(LVM)          then rc=configLine("BASEDEV="fileName(LVMDMDFILE)       LVMOPTIONS)
    
    rc=configLine("BASEDEV="fileName(BOOTFLTFILE) BOOTOPTIONS)
    
    /* Codepages and keyboard layout */
    rc=configLine("COUNTRY="COUNTRY",\os2\country.sys")
    rc=configLine("CODEPAGE="CODEPAGE)
    rc=configLine("DEVINFO=KBD,"KEYBOARD",\os2\keyboard.dcp")
    rc=configLine("DEVINFO=SCR,VGA,\os2\viotbl.dcp")
    
    /* Unicode */
    if isSet(UNICODE) then do
      rc=configLine("SET LANG="UNILANG)
      rc=configLine("SET ULSPATH=\language")
      rc=configLine("DEVICE=\os2\unicode.sys")
    end
    
    /* Device Drivers */
    if isSet(CDROM) then rc=configLine("DEVICE=\os2\"fileName(CDROMDMDFILE))
    if isSet(ASPI)  then rc=configLine("DEVICE=\os2\"fileName(ASPIFILE))
    
    /* Test device driver (required?) */
    rc=configLine("DEVICE="DRIVELETTER"\os2\testcfg.sys")
    
    /* Mouse support */
    if isSet(MOUSE) then do
        rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(MOUSEPOINTDDFILE))
        if MOUSEDEVFILE<>"" then rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(MOUSEDEVFILE) MOUSEDEVFILEOPTIONS)
        rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(MOUSEFILE) MOUSEFILEOPTIONS)
    end
    
    /* USB */
    if isSet(USB) then do
        do USBOHCI
            rc=configLine("BASEDEV="fileName(USBOHCIFILE) USBOHCIOPTIONS)
        end
        do USBUHCI
            rc=configLine("BASEDEV="fileName(USBUHCIFILE) USBUHCIOPTIONS)
        end
        do USBEHCI
            rc=configLine("BASEDEV="fileName(USBEHCIFILE) USBEHCIOPTIONS)
        end
        rc=configLine("BASEDEV="fileName(USBDFILE)   USBDOPTIONS)
        rc=configLine("BASEDEV="fileName(USBHIDFILE) USBHIDOPTIONS)
        if isSet(USBKEYB)    then rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(USBKEYBFILE)    USBKEYBOPTIONS)
        if isSet(USBCOM)     then rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(USBCOMFILE)     USBCOMOPTIONS)
        if isSet(USBMOUSE)   then rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(USBMOUSEFILE)   USBMOUSEOPTIONS)
        if isSet(USBPRINTER) then rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(USBPRINTERFILE) USBPRINTEROPTIONS)
        if isSet(USBCDROM)   then rc=configLine("BASEDEV="fileName(USBCDROMFILE)   USBCDROMOPTIONS)
        if isSet(USBSTORAGE) then rc=configLine("BASEDEV="fileName(USBSTORAGEFILE) USBSTORAGEOPTIONS)
    end
    
    /* Serial port support */
    if isSet(SERIALPORT) then do
        rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(SERIALPORTFILE) SERIALPORTOPTIONS)
    end
    
    /* Parallel port support */
    if isSet(PARALLELPORT) then do
        rc=configLine("DEVICE="DRIVELETTER"\os2\"fileName(PARALLELPORTFILE) PARALLELPORTOPTIONS)
    end
    
    /* Settings */
    rc=configLine("MEMMAN=NOSWAP,PROTECT")
    rc=configLine("IOPL=YES")
    rc=configLine("REIPL=ON")
    rc=configLine("PROTECTONLY=YES")
    rc=configLine("BUFFERS=32")
    rc=configLine("THREADS=96")
    rc=configLine("DISKCACHE=48,LW")
    rc=configLine("MAXWAIT=1")
    
    /* Command environment */
    rc=configLine("PROTSHELL="DRIVELETTER"\os2\"fileName(PROTSHELLFILE) PROTSHELLFILEOPTIONS)
    rc=configLine("SET OS2_SHELL="DRIVELETTER"\os2\"fileName(SHELLFILE) SHELLFILEOPTIONS)
    rc=configLine("SET COMSPEC="DRIVELETTER"\os2\"fileName(SHELLFILE))
    rc=configLine("SET PROMPT=$P$$$s")
    rc=configLine("SET KEYS=ON")
    rc=configLine("LIBPATH=.;a:\;a:\os2\dll;s:\os2\dll")
    rc=configLine("SET PATH=a:\;a:\os2;s:\os2;")
    rc=configLine("SET DPATH=a:\;a:\os2;s:\os2;")
    
    /* Additional config.sys lines */
    rc=configLine(ADDITIONALCFG)
    
    /* Close config.sys */
    rc=lineout(VIRTUALDRIVE"\config.sys")
    
    return rc
